<?php

namespace Drupal\sender\Entity;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Defines the interface of a message entity.
 */
interface MessageInterface extends ConfigEntityInterface {

  /**
   * Getter for the message's subject.
   *
   * @return string
   *   The message's subject.
   */
  public function getSubject();

  /**
   * Setter for the message's subject.
   *
   * @param string $subject
   *   The message's subject.
   */
  public function setSubject($subject);

  /**
   * Getter that returns only the format of the message's body.
   *
   * @return string
   *   The format of the body.
   */
  public function getBodyFormat();

  /**
   * Getter that returns only the value of the message's body.
   *
   * @return string
   *   The value of the body.
   */
  public function getBodyValue();

  /**
   * Getter for the message's body.
   *
   * @return array
   *   An associative array with 'value' and 'format' keys.
   */
  public function getBody();

  /**
   * Setter for the message's body.
   *
   * @param array $body
   *   An associative array with 'value' and 'format' keys.
   */
  public function setBody(array $body);

  /**
   * Returns a list of token types allowed in the message's body and subject.
   *
   * @return array
   *   A list of allowed token types.
   */
  public function getTokenTypes();

  /**
   * Sets allowed token types.
   *
   * @param array $token_types
   *   A list of token types.
   */
  public function setTokenTypes(array $token_types);

  /**
   * Builds a render array for the message.
   *
   * @param \Drupal\Core\Session\AccountInterface $recipient
   *   The recipient of the message.
   * @param array $data
   *   An array of values to be used in token replacement.
   *
   * @see \Drupal\Core\Utility\Token::replace()
   *
   * @return array
   *   A render array for the message.
   */
  public function build(AccountInterface $recipient, array $data);

}
