<?php

namespace Drupal\sender\Form;

use Drupal\sender\Entity\Message;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * A form to create and edit messages.
 */
class MessageForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // The message's subject.
    $form['subject'] = [
      '#type' => 'textfield',
      '#title' => t('Subject'),
      '#description' => t("The message's subject."),
      '#maxlength' => 255,
      '#default_value' => $this->entity->getSubject(),
      '#element_validate' => ['token_element_validate'],
      '#token_types' => $this->entity->getTokenTypes(),
      '#required' => TRUE,
    ];

    // The message's ID (machine name).
    $form['id'] = [
      '#type' => 'machine_name',
      '#maxlength' => Message::ID_MAX_LENGTH,
      '#default_value' => $this->entity->id(),
      '#disabled' => !$this->entity->isNew(),
      '#machine_name' => [
        'source' => ['subject'],
        'exists' => [$this, 'messageExists'],
      ],
    ];

    // The message's body.
    $form['body'] = [
      '#type' => 'text_format',
      '#title' => t('Body'),
      '#format' => $this->entity->getBodyFormat(),
      '#default_value' => $this->entity->getBodyValue(),
      '#element_validate' => ['token_element_validate'],
      '#token_types' => $this->entity->getTokenTypes(),
    ];

    // Shows a list of available tokens.
    $form['token_tree'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => $this->entity->getTokenTypes(),
      '#show_restricted' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // Shows success or error message.
    if (parent::save($form, $form_state)) {
      drupal_set_message(t('The message %entity has been saved.', ['%entity' => $this->entity->label()]));
    }
    else {
      drupal_set_message(t('An error occured.', 'error'));
    }

    // Redirects to the messages listing.
    $form_state->setRedirect('entity.sender_message.collection');
  }

  /**
   * Checks if a message with the given ID already exists.
   *
   * This method is used to check if a machine name was already used.
   */
  public function messageExists($id) {
    return (bool) Message::load($id);
  }

}
